#!/bin/bash

rezip_one_file () {
    gz_name="$1"
    bz2_name=$(echo "$1" | /bin/sed -e 's/\.gz$/.bz2/' -e 's/\.gz\./.bz2./')

    /bin/gzip -t "$gz_name" || {
        echo "Bad gzip file '$gz_name'"
        return 1
    }
    echo "Recompressing '$gz_name' as '$bz2_name'"
    /bin/gzip -dc "$gz_name" | /bin/bzip2 -zc >$bz2_name
    echo "Comparing resulting '$bz2_name' to '$gz_name'"
    gz_sum=$(/bin/gzip   -dc "$gz_name"  | /usr/bin/md5sum)
    bz2_sum=$(/bin/bzip2 -dc "$bz2_name" | /usr/bin/md5sum)
    if [ "$gz_sum" = "$bz2_sum" ] ; then
        echo "Recompression succeeded"
        /bin/rm -f "$gz_name"
        return 0
    else
        echo "Recompression failed"
        /bin/rm -f "$bz2_name"
        return 1
    fi
}

for f in "$@" ; do
    if [ -f "$f" ] ; then
        echo "Processing '$f'"
        case "$f" in
        *.bz2|*.bz2.*)
            echo "Skipping '$f', since it already looks like a bzip2 file"
            ;;
        *.gz|*.gz.*)
            rezip_one_file "$f"
            ;;
        *)
            /bin/bzip2 "$f"
            ;;
        esac
    fi
done
