#!/bin/bash

source /usr/local/bin/knoppmyth_functions.bash

DOPTS=(--backtitle "$BT" --title "LIRC Configuration" --trim --aspect 10 )

init () {
  REMPATH=/usr/local/share/knoppmyth/remotes
  BLASTERPATH=/usr/local/share/knoppmyth/irblaster
  RCRC=/home/mythtv/.mythtv/lircrc
  LIRC=/etc/lirc
  ATI=/lib/modules/$(uname -r)/kernel/drivers/input/misc
  maintainer="knoppmyth@gmail.com"
  logfile=/var/log/mythtv/lirc-reconfig.log
  detected=0
  should_have_detected_remote=0
  remote=""
  NOT_LISTED="My remote is not listed"
  NO_REMOTE="No Remote"
}

check_for_multiple_remotes() {
  if [ "$remote" ]; then # there's already been another remote detected
    dialog "${DOPTS[@]}" --msgbox "You seem to have more than
      one detectable USB remote on your system. You'll need to choose your 
      remote from the following menu, or you can disconnect the remote(s)
      you do not want to use then run $0 again." 0 0
    remote=""
    return 1
  fi

  return 0
}

# Parse /proc/bus/usb/devices, looking for the telltale signs of
# known USB remotes. As it only makes sense to configure one USB 
# remote at a time, if we detect more than one, inform the user that
# they will have to use the full menu or run the program again after
# removing all but one remote.

look_for_usb_remotes() {
  detected=0

    if [ -f /proc/bus/usb/devices ]; then
       if [ `grep -c -e "^P:  Vendor=0bc7 ProdID=0008 Rev= 1.00" /proc/bus/usb/devices` == 1 ]; then # SnapStream Firefly
       remote=SnapStream_Firefly
       detected=1
       fi
    fi

    if [ `grep -c -e "^P:  Vendor=10c4 ProdID=0003 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # CommandIR Transceiver
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
     	remote=CommandIR_Tranceiver
    	detected=1
       fi
    fi

    if [ `grep -c -e "^P:  Vendor=0bc7 ProdID=0004 Rev= 1.00" /proc/bus/usb/devices` == 1 ]; then # ATI Remote Wonder
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
      	remote=ATI_Remote_Wonder
   	detected=1
       fi
    fi
	
    if [ `grep -c -e "^P:  Vendor=0bc7 ProdID=0004 Rev= 2.00" /proc/bus/usb/devices` == 1 ]; then # ATI Remote Wonder Plus
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
#      remote=ATI_Remote_Wonder_Plus
      	remote=ATI_Remote_Wonder
   	detected=1
       fi
    fi
    
    if [ `grep -c -e "^P:  Vendor=0e9c ProdID=0000 Rev= 1.00" /proc/bus/usb/devices` == 1 ]; then # Streamzap Remote:
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=StreamZap
       detected=1
       fi
    fi
	
    if [ `grep -c -e "^P:  Vendor=0fe9 ProdID=9010 Rev= 1.00" /proc/bus/usb/devices` == 1 ]; then # DVICO Remote:
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
	remote=Dvico
       detected=1
       fi
    fi
	
    if [ `grep -c -e "^P:  Vendor=6253 ProdID=0100 Rev= 1.00" /proc/bus/usb/devices` == 1 ]; then # VisionPlus Remote:
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
	return
       else
	remote=Twinhan_USB # the VisionPlus is a Twinhan clone
       detected=1
       fi
    fi

#+++++++ 25 +++++++++++++++++++++++++++++++ MCE USB2-new Remote +++++++ Start +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    if [ `grep -c -e "^P:  Vendor=1509 ProdID=9242 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # FIC eHome Infrared Transceiver 1509/9242
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi    
   
    if [ `grep -c -e "^P:  Vendor=1934 ProdID=0602 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # Fintek eHome Infrared Transceiver 1934/0602
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi

    if [ `grep -c -e "^P:  Vendor=147a ProdID=e015 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # Formosa eHome Infrared Transceiver 147a/e015
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi

    if [ `grep -c -e "^P:  Vendor=147a ProdID=e017 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # Formosa aim / Trust MCE Infrared Receiver 147a/e017
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi

    if [ `grep -c -e "^P:  Vendor=147a ProdID=e018 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # Formosa Industrial Computing 147a/e018
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi

    if [ `grep -c -e "^P:  Vendor=107b ProdID=3009 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # Gateway eHome Infrared Transceiver 107b/3009
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
	fi
    fi

    if [ `grep -c -e "^P:  Vendor=195d ProdID=7002 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # Itron ione Libra Q-11 195d/7002
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi

    if [ `grep -c -e "^P:  Vendor=043e ProdID=9803 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # LG eHome Infrared Transceiver 043e/9803
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi

    if [ `grep -c -e "^P:  Vendor=045e ProdID=00a0 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # Microsoft MCE Infrared Transceiver 045e/00a0
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi

    if [ `grep -c -e "^P:  Vendor=03ee ProdID=2501 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # Mitsumi 03ee/2501
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi

    if [ `grep -c -e "^P:  Vendor=0471 ProdID=060c Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # Philips Infrared Transciever - HP branded 0471/060C
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi

    if [ `grep -c -e "^P:  Vendor=0471 ProdID=060d Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # Philips SRM5100 0471/060d
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi
  
    if [ `grep -c -e "^P:  Vendor=0471 ProdID=0815 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # Philips eHome Infrared Transciever 0471/0815
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi
  
    if [ `grep -c -e "^P:  Vendor=2304 ProdID=0225 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # Pinnacle Systems, Inc. 2304/0225
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi

    if [ `grep -c -e "^P:  Vendor=179d ProdID=0010 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # Ricavision internal Infrared Transceiver 179d/0010
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi

    if [ `grep -c -e "^P:  Vendor=1308 ProdID=c001 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # Shuttle eHome Infrared Transceiver 1308/c001
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi

    if [ `grep -c -e "^P:  Vendor=051c ProdID=c001 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # Shuttle2 eHome Infrared Transceiver 051c/c001
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi

    if [ `grep -c -e "^P:  Vendor=0609 ProdID=031d Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # SMK/Toshiba G83C0004D410 0609/031d
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi

    if [ `grep -c -e "^P:  Vendor=0609 ProdID=0322 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # SMK eHome Infrared Transceiver (Sony VAIO) 0609/0322
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi

    if [ `grep -c -e "^P:  Vendor=0609 ProdID=0334 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then #  SKM bundled with Hauppauge PVR-150 0609/0334
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi

    if [ `grep -c -e "^P:  Vendor=1460 ProdID=9150 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # Tatung eHome Infrared Transceiver 1460/9150
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi

    if [ `grep -c -e "^P:  Vendor=1784 ProdID=0001 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # Topseed eHome Infrared Transceiver 1784/0001
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi

    if [ `grep -c -e "^P:  Vendor=1784 ProdID=0006 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # Topseed HP eHome Infrared Transceiver 1784/0006
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi

    if [ `grep -c -e "^P:  Vendor=1784 ProdID=0007 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # Topseed HP eHome Infrared Transceiver 1784/0007
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi


    if [ `grep -c -e "^P:  Vendor=1784 ProdID=0008 Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # Topseed eHome Infrared Transceiver 1784/0008
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-new
       detected=1
       fi
    fi
#+++++++++++++++++++++++++++++++++++++++++ MCE USB2-new Remote ++++++ End +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#+++++++++ 1 +++++++++++++++++++++++++++++ MCE USB2-old Remote +++++ Start ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    if [ `grep -c -e "^P:  Vendor=045e ProdID=006d Rev= 0.00" /proc/bus/usb/devices` == 1 ]; then # USB Microsoft IR Transceiver Driver 045e/006d
       check_for_multiple_remotes
       if [ $? == 1 ]; then # there was already a remote detected
       return
       else
       remote=MCEUSB-old
       detected=1
       fi
    fi
#+++++++++++++++++++++++++++++++++++++++++ MCE USB2-old Remote ++++++ End ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
}

# Call this before configuring a remote that should have been detected 
# automatically. If it was not detected automatically, this asks the 
# user to send us some data we can use to build in detection.
# parameter1: name of remote

remote_was_detectable() {
  if [ "$detected" == 0 ]; then
    dialog "${DOPTS[@]}" --title "WARNING WARNING WARNING WARNING WARNING" \
      --msgbox "I should have been able to auto-detect this remote. If 
      you're sure it's plugged in, please send the contents of $logfile
      to $maintainer. Otherwise, please be sure to plug the remote in
      and re-run this script." 0 0
    cat /proc/bus/usb/devices > $logfile
    echo "$0:" >> $logfile
    echo "$0: The $1 remote should be in there somewhere." >> $logfile
    echo "$0:" >> $logfile
  fi
}

no_details_for_remote() {
  dialog "${DOPTS[@]}" --msgbox "Sorry, I don't have details on 
    setting up your remote. If you work out how to set up your remote, please
    contact $maintainer with the details so that it
    can be supported in future releases of KnoppMyth. NOTE: You should only
    contact $maintainer with details of how to get it working. If you can't get
    your remote working, use the forum for support or check the docs at
    http://lirc.org." 0 0
}

sorry_no_lircrc() {
  dialog "${DOPTS[@]}" --msgbox "While the module and lircd.conf have
    been setup, I don't have a lircrc file for your remote. Please contact
    $maintainer, if you have one." 0 0
}

setting_up_remote() {
  dialog "${DOPTS[@]}" --infobox "Setting up $remote remote..." 0 0
  sleep 2.0
}

needs_lircd_for_your_remote () {
  dialog "${DOPTS[@]}" --msgbox "LIRC has been configured for use 
    with a $remote but you still needs an appropriate lircd.conf 
    file in /etc/lirc. By default, a lircd.conf file for the Hauppauge
    remote has been installed. If this is not what you want, you can
    either generate the file by running \"irrecord -d /dev/lirc0\" or
    obtain an appropriate file from http://lirc.org. You will then need
    to write a lircrc file and place it in ~mythtv/.mythtv/lircrc.
    Once you have a working setup you should send your lircrc and
    lircd.conf to $maintainer so that you can be given the option of
    selecting your remote from a menu in future versions of KnoppMyth." 0 0
}

select_blasted_device () {
  BLASTED_DEVICE_NOT_LISTED="My Device is not Listed"
  unset blasted_devices
  declare -a blasted_devices

  for device in $(ls $BLASTERPATH) ; do
    blasted_devices=( "${blasted_devices[@]}" $device "" )
  done

  if ! dialog "${DOPTS[@]}" --menu "Select the device controlled by your
         blaster:" 0 0 0 \
         "${blasted_devices[@]}" \
         "$BLASTED_DEVICE_NOT_LISTED" "" 2> /tmp/blasted_device ; then
    echo Abort
    exit
  fi

  blasted_device=$(cat /tmp/blasted_device)
	
  if [ "$blasted_device" = "$BLASTED_DEVICE_NOT_LISTED" ] ; then
    dialog "${DOPTS[@]}" --msgbox "You need to append the lircd.conf file
      containing the remote codes for the device you wish to control with
      your IR blaster to /etc/lirc/lircd.conf. Please send the lircd.conf
      file for the device you wish to control to $maintainer so that it
      can be included in future releases of KnoppMyth." 10 70
  else
    cat $BLASTERPATH/$blasted_device/irblasterd.conf >> \
      /etc/lirc/lircd.conf
  fi		
}

remote_menu () {
  clear

  look_for_usb_remotes # try to detect USB remotes automatically

  echo "Found a $remote" >> /tmp/remotes_found.txt

  # If we auto-detected zero or multiple USB remotes, present a menu for the 
  # user to select a remots.

  if [ "$remote" == "" ]; then 

    declare -a remote_selections

    for remote in $(ls $REMPATH) ; do
      remote_selections=( "${remote_selections[@]}" $remote "" )
    done

    if ! dialog "${DOPTS[@]}" --menu "Select Your Remote:" 20 66 14 \
	       "${remote_selections[@]}" \
               "$NOT_LISTED" "" \
	       "$NO_REMOTE" "" 2> /tmp/selected_remote ; then
      echo Abort
      exit
    fi

    remote=$(cat /tmp/selected_remote)
  fi

  echo "selected remote is $remote"
  echo "remote=$remote">/home/mythtv/.mythtv/smolt.info
  if [ "$remote" = "$NOT_LISTED" ] ; then
    no_details_for_remote
    exit    
  elif [ "$remote" = "$NO_REMOTE" ] ; then
    exit
  fi

  # Check if that remote is one that should have been detectable.

  if [ -e $REMPATH/$remote/detectable ] ; then
    remote_was_detectable $remote
  fi

  # If the ATI Remote Wonder remote was selected, get confirmation before
  # renaming the ati_remote kernel module.

  if [ $remote = "ATI_Remote_Wonder" ] ; then
    if ! dialog "${DOPTS[@]}" --title "ATI Remote Wonder" --yesno "This
        will remove the kernel module and use LIRC instead. Do you wish to
        continue?" 0 0 ; then
      dialog "${DOPTS[@]}" --msgbox "LIRC Setup aborted." 0 0
      exit
    fi

    rmmod ati_remote
    mv $ATI/ati_remote.ko $ATI/ati_remote.NOko
  fi

  # Copy the configuration file from the remote directory to /etc/lirc

  if [ ! -e $REMPATH/$remote/hardware.conf -o \
       ! -e $REMPATH/$remote/lircd.conf ] ; then
    no_details_for_remote
    exit
  fi

  setting_up_remote

  cp -f $REMPATH/$remote/hardware.conf $LIRC/
  cp -f $REMPATH/$remote/lircd.conf $LIRC/

  if [ -e $REMPATH/$remote/lircmd.conf ] ; then
    cp -f $REMPATH/$remote/lircdm.conf $LIRC/
  fi

  if [ -e $REMPATH/$remote/lircrc ] ; then
    cp -f $REMPATH/$remote/lircrc $RCRC
    chown mythtv.mythtv $RCRC
  else
    sorry_no_lircrc
  fi

  if [ $remote = "Hauppauge_PVR-150_with_transmitter_and_receiver-dog_bone" -o $remote = "Hauppauge_PVR-150_with_transmitter_and_receiver" ] ; then
    dialog "${DOPTS[@]}" --msgbox "You may still have to do some 
      manual configuration for the 150's \"IR-Blaster\". Please contact
      $maintainer and provide details to further automate this process." 0 0
  fi

  if [ $remote = "MCEUSB-new" ] ; then
     dialog --title "MCEUSB-new Remote Control" --msgbox "NOTICE OF CHANGE: This version of KnoppMyth will now support the 
MCE Remote and Media Center Edition Remote Keyboard.
     
To support both the remote and keyboard the module lirc_mceusb2
has been built into the lirc_mod_mce module. For this reason the 
lirc_mod_mce module is a complete replacement for the lirc_mceusb2
module. The lirc_mceusb2 module is no longer used and loading both 
of these modules at the same time will cause problems. 
See www.knoppmythwiki.org   Media_Center_Edition_Remote Control
for more information." 14 72
# Note: If you add spaces before the dialog text above, it throws the text box into turmoil.
  fi
  
  #if [ $remote = "MCEUSB-new" ] ; then
  #  dialog "${DOPTS[@]}" --title "Warning" --msgbox "NOTE, the included
  #    lircrc is for the older MCEUSB and may not work. If you have an
  #    updated version, please send it to $maintainer" 0 0
  #fi

  if [ $remote = "ATI_Remote_Wonder_Plus" ] ; then
    dialog "${DOPTS[@]}" --title "Warning" --msgbox "NOTE, the included
      lircrc only has bindings for MythTV.  You'll need to update it for
      Mplayer and Xine.  Please send it to $maintainer" 0 0
  fi

  if [ $remote = "RCA-6N1" ] ; then
    echo "setserial /dev/ttyS0 uart none" >> /etc/init.d/bootmisc.sh
    setserial /dev/ttyS0 uart none
  fi

  if [ $remote = "Twinhan_USB" ] ; then
    no_details_for_remote
    exit
  fi

  if [ $remote = "CommandIR_Tranceiver" ] ; then
    needs_lircd_for_your_remote
    select_blasted_device
  fi

  if [ $remote = "Homebrew_Serial_Receiver" ] ; then
    echo "setserial /dev/ttyS0 uart none" >> /etc/init.d/bootmisc.sh
    setserial /dev/ttyS0 uart none

    needs_lircd_for_your_remote

    if dialog "${DOPTS[@]}" --yesno "Do you have an IR Blaster on the 
         same serial port?" 0 0 ; then
      select_blasted_device
    fi
  fi
  
  # Finally restart lirc with the new configuration in place.

  /etc/init.d/lirc restart
}

must_be_root
init
remote_menu
exit 0

# End
