#!/bin/bash

fatal () {
    echo 1>&2 "Error! $*"
    exit 1
}

. /usr/local/bin/backupcommon ||
    fatal "Can not load common settings!"

# Prevent mythshutdown from shutting down the system in the middle...
/usr/bin/mythshutdown --lock

if [ "$1" = '-b' ] ; then # Optionally, make a backup before we start...
    # I'm not sure how useful this is, since both recover and analyze
    # are pretty safe operations...
    /usr/local/bin/mythbackup ||
        fatal "Backup failed! Aborting!"
fi
# The database needs to be idle when we do this...
# Having it auto restarted would also be "bad". ;-)
/etc/init.d/mythtv-backend stop
/etc/init.d/mysql stop

cd $DATABASE_DIR || fatal "Could not cd to $DATABASE_DIR"

for tbl in *.MYI ; do
    # If the fast way fails fall back on the old slow way.
    /usr/bin/myisamchk --recover $tbl || {
        echo "Fast recovery of $tbl failed, attempting slower safe recovery..."
        /usr/bin/myisamchk --safe-recover $tbl ||
            fatal "Even safe recovery of $tbl failed! Aborting!"
    }
    /usr/bin/myisamchk --analyze $tbl
done

# Resatart the servers in the right order...
/etc/init.d/mysql stop ; /etc/init.d/mysql start
# The restart option of mythtv-backend can fail if the server isn't running
/etc/init.d/mythtv-backend stop ; /etc/init.d/mythtv-backend start

# Unlock the system again...
/usr/bin/mythshutdown --unlock
