#!/bin/bash

PYTHONPATH=/usr/local/bin /usr/bin/python <<'EOF'

import os, file_editor

apps = (
    ("xscreensaver", "#[startup] {xscreensaver}\n"),
    ("x11vnc", "#[startup] {/usr/bin/x11vnc -nap -wait 50 -passwd mythtv -display :0 -forever -o /var/log/x11vnc.log -bg}\n"),
    ("gnome-volume-manager", "[startup] {gnome-volume-manager}\n"),
    ("xvattr", "[startup] {xvattr -a XV_COLORKEY -v 0}\n"),
)
ed = file_editor.file_editor("/home/mythtv/.fluxbox/apps")
ed.set_pos(ed.find("[startup] {KnoppMyth-run}"))
for pattern, line in apps:
    if ed.find(pattern) is None:
        ed.insert(line)
ed.close()

ed = file_editor.file_editor("/home/mythtv/.fluxbox/init")
ed.replace("styles/Clean", "styles/Flux")
ed.close()

fname = "/home/mythtv/.mplayer/config"
if os.path.exists(fname):
    ed = file_editor.file_editor(fname)
#    ed.re_remove("^heartbeat-cmd=.*$")
    if not ed.re_find("^heartbeat-cmd=.*$"):
    	ed.append('\nheartbeat-cmd="/usr/bin/xscreensaver-command -deactivate >/dev/null"\n')
    ed.close()

for fname in ("/etc/mythtv/modules/ivtv", "/etc/modprobe.conf"):
    if not os.path.exists(fname):
    	continue
    ed = file_editor.file_editor(fname)
    ed.re_remove("^options tuner.*$")
    ed.re_remove("^options saa7127.*$")
    ed.re_replace("^(options msp3400.*) simple=1", "\\1")
    ed.re_replace("^(options ivtv .*) (debug=[0-9]+)", "\\1")
    ed.re_replace("^(options ivtv .*) (yuv_buffers=[0-9]+)", "\\1 enc_\\2")
    ed.re_replace("^(options ivtv .*) (mpg_buffers=[0-9]+)", "\\1 enc_\\2")
    ed.re_replace("^(options ivtv .*) (vbi_buffers=[0-9]+)", "\\1 enc_\\2")
    ed.re_replace("^(options ivtv .*) (pcm_buffers=[0-9]+)", "\\1 enc_\\2")
    # If we made the ivtv options line empty, remove it.
    ed.re_replace("^options ivtv *$", "")
    ed.close()

# The following fixes address a X performance problem with the nvidia driver

fname = "/home/mythtv/.nvidia-settings-rc"
if os.path.exists(fname):
    ed = file_editor.file_editor(fname)
    ed.re_replace("^([0-9]/XVideoTextureSyncToVBlank)=.*", "\\1=0")
    ed.re_replace("^([0-9]/XVideoBlitterSyncToVBlank)=.*", "\\1=0")
    ed.close()

xorg = (
    ('XvmcUsesTextures', '\tOption "XvmcUsesTextures" "false"\n'),
    ('UseEvents', '\tOption "UseEvents" "true"\n'),
    ('NVAGP', '\t#Option "NVAGP" "2"\n'),
)
ed = file_editor.file_editor("/etc/X11/xorg.conf")
begin_section = ed.re_find('^[ \t]*Section[ \t]*"Device"')
nvidia_driver=0
while begin_section is not None:
    end_section = ed.re_find("^[ \t]*EndSection", begin_section)
    driver_line = ed.re_find('^[ \t]*Driver[ \t]*"nvidia"', begin_section)
    if end_section is not None and driver_line is not None and driver_line < end_section:
        nvidia_driver=1
        ed.set_pos(end_section)
	for pattern, line in xorg:
            found = ed.find(pattern, begin_section)
            if found is None or found > end_section:
                ed.insert(line)
    begin_section = ed.re_find('^[ \t]*Section[ \t]*"Device"', begin_section+1)
if nvidia_driver and not ed.re_find('"Composite"[ \t]*"Disabled"'):
    ed.append('\n# Needed for XvMC with some Nvidia boards and drivers\n')
    ed.append('#Section "Extensions"\n#\tOption "Composite" "Disabled"\n#EndSection\n')
ed.close()

#Strip extra unused workspaces to minimize key conflicts

for fname in ("/etc/X11/fluxbox/keys", "/home/mythtv/.fluxbox/keys"):
    if not os.path.exists(fname):
    	continue
    ed = file_editor.file_editor(fname)
    ed.re_remove("^Mod1[ \t]*F[3-9][ \t]*:Workspace[3-9][ \t]*\n")
    ed.re_remove("^Mod1[ \t]*F1[0-9][ \t]*:Workspace1[0-9][ \t]*\n")
ed.close()

# This one is a doozy... Note we're only opening /etc/mythtv/mysql.txt to
# get the hostname (which we give precendence) and don't make any changes
# unless the other two files are inconsistent.

p0 = '^DBHostName='
p1 = '[A-Za-z0-9.-]+'  # Matches a hostname
p2 = '^[ \t]*setenv[ \t]+db_server[ \t]+"'
p3 = '^[ \t]*bind-address[ \t]*=[ \t]*'

noed = file_editor.file_editor("/etc/mythtv/mysql.txt")
hostbeg = noed.re_find(p0, 0, file_editor.AFTER)
hostend = noed.re_find(p1, hostbeg, file_editor.AFTER)
hostname = noed.text[hostbeg:hostend].strip()
noed.close()

if hostname.lower() not in ('localhost', '127.0.0.1'):
    # OK, so we need to make sure everything else matches...
    ed = file_editor.file_editor("/etc/mythtv/mythweb-htaccess")
    if ed.re_find(p2 + hostname + '"') is None:
        ed.re_replace('(' + p2 + ')' + p1 + '(")', '\\1' + hostname + '\\2')
    ed.close()

    ed = file_editor.file_editor("/etc/mysql/my.cnf")
    if ed.re_find(p3 + hostname) is None:
        ed.re_replace('(' + p3 + ')' + p1, '\\1' + hostname)
    ed.close()

# Add new edits BEFORE this line!!!
EOF

# Make sure everything ends up with the right owners...
chown -R mythtv:mythtv /home/mythtv
