#!/usr/bin/perl
#
#  rrd_MBfan.pl
#	Motherboard fan data collection routine for KnoppMyth
#
########################################################################
# Configuration:
my $dbf = 'MBfan';
my $configfile = '/etc/rrd.config';
########################################################################
use RRDs;

if (! -d "/myth") { $configfile = "./D_rrd.config"; }	# DEBUG
do $configfile;

sub create {
    #	$_[0] = filename
    if (! -e "$log/$_[0].rrd") {
	print "Create db for $_[0] => $log/$_[0].rrd\n";
	RRDs::create( "$log/$_[0].rrd", "-s 300",
	    "DS:mbf1:GAUGE:600:0:U",
	    "DS:mbf2:GAUGE:600:0:U",
	    "DS:mbf3:GAUGE:600:0:U",
	    "RRA:AVERAGE:0.5:1:576",
	    "RRA:AVERAGE:0.5:6:672",
	    "RRA:AVERAGE:0.5:24:732",
	    "RRA:AVERAGE:0.5:144:1460");
	$ERROR = RRDs::error;
	print "Error: RRDs::create failed for '$_[0]' : $ERROR\n" if $ERROR;
    }
}

my ($mbf1, $mbf2, $mbf3);

sub gather {
    $mbf1 = 0;  # Motherboard fan
    $mbf2 = 0;  # CPU fan
    $mbf3 = 0;  # Case fan
    if ($MBF_mobo ne "") {
        $mbf1 = `$MBF_prog $MBF_mobo`;
    }
    if ($MBF_cpu ne "") {
        $mbf2 = `$MBF_prog $MBF_cpu`;
    }
    if ($MBF_case ne "") {
        $mbf3 = `$MBF_prog $MBF_case`;
    }
    $mbf1 =~ s/[\n ]//g;
    $mbf2 =~ s/[\n ]//g;
    $mbf3 =~ s/[\n ]//g;
    print "$dbf: motherboard $mbf1, CPU $mbf2, case $mbf3, rpm\n";
}

sub update {
    #	$_[0] = filename
    RRDs::update( "$log/$_[0].rrd", "-t",
	"mbf1:mbf2:mbf3",
	"N:$mbf1:$mbf2:$mbf3");
    $ERROR = RRDs::error;
    print "Error: RRDs::update for '$_[0]' : $ERROR\n" if $ERROR;
}

sub graph {
    #	$_[0] = time interval (ie: day...)
    #	$_[1] = filename suffix.
    undef @args;
    @args = ( "$png/$dbf-$_[1].png", "-s -1$_[0]", "-aPNG",
	"-w $Gwd", "-h $Ght", "-E", "-l 20", "-M",
	"--color", "SHADEA$color",
	"--color", "SHADEB$color",
	"--color", "BACK$color",
	"-t Motherboard, CPU & Case fan speeds rpm :: $_[0]");
    if ($MBF_mobo ne "") {
        push @args, "DEF:mbf1=$log/$dbf.rrd:mbf1:AVERAGE",
	"LINE1:mbf1$color_mbf1: MB fan\\:",
	"GPRINT:mbf1:MIN:Minimum\\: % 5.1lf",
	"GPRINT:mbf1:MAX:Maximum\\: % 5.1lf",
	"GPRINT:mbf1:AVERAGE:Average\\: % 5.1lf",
	"GPRINT:mbf1:LAST:Current\\: % 5.1lf rpm\\j";
    }
    if ($MBF_cpu ne "") {
	push @args, "DEF:mbf2=$log/$dbf.rrd:mbf2:AVERAGE",
	"LINE1:mbf2$color_mbf2:CPU fan\\:",
	"GPRINT:mbf2:MIN:Minimum\\: % 5.1lf",
	"GPRINT:mbf2:MAX:Maximum\\: % 5.1lf",
	"GPRINT:mbf2:AVERAGE:Average\\: % 5.1lf",
	"GPRINT:mbf2:LAST:Current\\: % 5.1lf rpm\\j";
    }
    if ($MBF_case ne "") {        
	push @args, "DEF:mbf3=$log/$dbf.rrd:mbf3:AVERAGE",
	"LINE1:mbf3$color_mbf3:Case fan\\:",
	"GPRINT:mbf3:MIN:Minimum\\: % 5.1lf",
	"GPRINT:mbf3:MAX:Maximum\\: % 5.1lf",
	"GPRINT:mbf3:AVERAGE:Average\\: % 5.1lf",
	"GPRINT:mbf3:LAST:Current\\: % 5.1lf rpm\\j";
     }
     RRDs::graph(@args);

    $ERROR = RRDs::error;
    print "Error: RRDs::graph failed for '$_[0]' : $ERROR\n" if $ERROR;
}
########################################################################
create "$dbf";
gather;
update "$dbf";
graph 'day',   'Daily';
graph 'week',  'Weekly';
graph 'month', 'Monthly';
graph 'year',  'Yearly';
########################################################################
# vim: sw=4 ts=8
# End
