#!/usr/bin/perl
#
#  rrd_MBtemp.pl
#	Motherboard Temperature data collection routine for KnoppMyth
#
########################################################################
# Configuration:
my $dbf = 'MBtemp';
my $configfile = '/etc/rrd.config';
########################################################################
use RRDs;

if (! -d "/myth") { $configfile = "./D_rrd.config"; }	# DEBUG
do $configfile;

sub create {
    #	$_[0] = filename
    if (! -e "$log/$_[0].rrd") {
	print "Create db for $_[0] => $log/$_[0].rrd\n";
	RRDs::create( "$log/$_[0].rrd", "-s 300",
	    "DS:mbt1:GAUGE:600:0:U",
	    "DS:mbt2:GAUGE:600:0:U",
	    "DS:mbt3:GAUGE:600:0:U",
	    "RRA:AVERAGE:0.5:1:576",
	    "RRA:AVERAGE:0.5:6:672",
	    "RRA:AVERAGE:0.5:24:732",
	    "RRA:AVERAGE:0.5:144:1460");
	$ERROR = RRDs::error;
	print "Error: RRDs::create failed for '$_[0]' : $ERROR\n" if $ERROR;
    }
}

my ($mbt1, $mbt2, $mbt3);

sub gather {
    $mbt1 = 0;
    $mbt2 = 0;
    $mbt3 = 0;
    if ($MBT_mobo ne "") {
        $mbt1 = `$MBT_prog $MBT_mobo`;
    }
    if ($MBT_cpu ne "") {
        $mbt2 = `$MBT_prog $MBT_cpu`;
    }
    if ($MBT_case ne "") {
        $mbt3 = `$MBT_prog $MBT_case`;
    }
    $mbt1 =~ s/[\n ]//g;
    $mbt2 =~ s/[\n ]//g;
    $mbt3 =~ s/[\n ]//g;
    print "$dbf: motherboard $mbt1, CPU $mbt2, case $mbt3, C\n";
    # The motherboard sensor occasionally returns nonsense values.
    # This should keep the spurious peaks from roaching the graph...
    if ($mbt1 > 100.0) { $mbt1 = $mbt2 }
    print "$dbf: motherboard $mbt1, CPU $mbt2, case $mbt3, C\n";
}

sub update {
    #	$_[0] = filename
    RRDs::update( "$log/$_[0].rrd", "-t",
	"mbt1:mbt2:mbt3",
	"N:$mbt1:$mbt2:$mbt3");
    $ERROR = RRDs::error;
    print "Error: RRDs::update for '$_[0]' : $ERROR\n" if $ERROR;
}

#
sub graph {
    #	$_[0] = time interval (ie: day...)
    #	$_[1] = filename suffix.
    undef @args;
    @args = ( "$png/$dbf-$_[1].png", "-s -1$_[0]", "-aPNG",
        "-w $Gwd", "-h $Ght", "-E", "-l 20", "-M",
        "--color", "SHADEA$color",
        "--color", "SHADEB$color",
        "--color", "BACK$color",
        "-t Motherboard & CPU temperature degrees C :: $_[0]");
    if ($MBT_mobo ne "") {
        push @args, "DEF:mbt1=$log/$dbf.rrd:mbt1:AVERAGE",
        "LINE1:mbt1$color_mbt1: MB temp\\:",
	"GPRINT:mbt1:MIN:Minimum\\: % 5.1lf",
	"GPRINT:mbt1:MAX:Maximum\\: % 5.1lf",
	"GPRINT:mbt1:AVERAGE:Average\\: % 5.1lf",
	"GPRINT:mbt1:LAST:Current\\: % 5.1lf C\\j";
    }
    if ($MBT_cpu ne "") {
        push @args, "DEF:mbt2=$log/$dbf.rrd:mbt2:AVERAGE",
	"LINE1:mbt2$color_mbt2:CPU temp\\:",
	"GPRINT:mbt2:MIN:Minimum\\: % 5.1lf",
	"GPRINT:mbt2:MAX:Maximum\\: % 5.1lf",
	"GPRINT:mbt2:AVERAGE:Average\\: % 5.1lf",
	"GPRINT:mbt2:LAST:Current\\: % 5.1lf C\\j";
    }
    if ($MBT_case ne "") {
        push @args, "DEF:mbt3=$log/$dbf.rrd:mbt3:AVERAGE",
	"LINE1:mbt3$color_mbt3:CASE temp\\:",
	"GPRINT:mbt3:MIN:Minimum\\: % 5.1lf",
	"GPRINT:mbt3:MAX:Maximum\\: % 5.1lf",
	"GPRINT:mbt3:AVERAGE:Average\\: % 5.1lf",
	"GPRINT:mbt3:LAST:Current\\: % 5.1lf C\\j";
    }
    RRDs::graph(@args);
    $ERROR = RRDs::error;
    print "Error: RRDs::graph failed for '$_[0]' : $ERROR\n" if $ERROR;
}
########################################################################
create "$dbf";
gather;
update "$dbf";
graph 'day',   'Daily';
graph 'week',  'Weekly';
graph 'month', 'Monthly';
graph 'year',  'Yearly';
########################################################################
# vim: sw=4 ts=8
# End
