#!/usr/bin/perl
#
#  rrd_UPS.pl
#	Uninterrupable power supply collection routine for KnoppMyth
#
########################################################################
# Configuration:
my $dbf = 'UPS';
my $configfile = '/etc/rrd.config';
$ups = 'mythups@localhost';
########################################################################
use RRDs;

if (! -d "/myth") { $configfile = "./D_rrd.config"; }	# DEBUG
do $configfile;

sub create {
    #	$_[0] = filename
    if (! -e "$log/$_[0].rrd") {
	print "Create db for $_[0] => $log/$_[0].rrd\n";
	RRDs::create( "$log/$_[0].rrd", "-s 300",
	    "DS:load:GAUGE:600:0:U",
	    "DS:input:GAUGE:600:0:U",
	    "DS:charge:GAUGE:600:0:U",
	    "RRA:AVERAGE:0.5:1:576",
	    "RRA:AVERAGE:0.5:6:672",
	    "RRA:AVERAGE:0.5:24:732",
	    "RRA:AVERAGE:0.5:144:1460");
	$ERROR = RRDs::error;
	print "Error: RRDs::create failed for '$_[0]' : $ERROR\n" if $ERROR;
    }
}

sub gather {
    my $status = '/var/log/apcupsd.status';
    if(-e "/bin/upsc") { # Network UPS Tools
        open F, "upsc $ups |";
        while(<F>) {
            if($_ =~ m/battery\.charge:\s+(\w+)/) {
                $charge = $1;
            }
            if($_ =~ m/input\.voltage:\s+(\w+)/) {
                $input = $1;
            }
            if($_ =~ m/ups\.load:\s+(\w+)/) {
                $load = $1;
            }
        }
        close F;
    } elsif(-e $status) { # apcupsd
        open F, $status;
        while(<F>) {
            if($_ =~ m/BCHARGE\s*:\s*(\d+\.\d+)\s+/) {
                $charge = $1;
            }
            if($_ =~ m/LINEV\s*:\s*(\d+\.\d+)\s+/) {
                $input = $1;
            }
            if($_ =~ m/LOADPCT\s*:\s*(\d+\.\d+)\s+/) {
                $load = $1;
            }
        }
        close F;
    }
    print "$dbf: load='$load' input='$input' charge='$charge'\n";
}

sub update {
    #	$_[0] = filename
    RRDs::update( "$log/$_[0].rrd", "-t",
	"load:input:charge",
	"N:$load:$input:$charge");
    $ERROR = RRDs::error;
    print "Error: RRDs::update for '$_[0]' : $ERROR\n" if $ERROR;
}

sub graph {
    #	$_[0] = time interval (ie: day...)
    #	$_[1] = filename suffix.
    RRDs::graph( "$png/$dbf-$_[1].png", "-s -1$_[0]", "-aPNG",
	"--alt-y-grid", "-w $Gwd", "-h $Ght", "-l 0", "-E",
	"--color", "SHADEA$color",
	"--color", "SHADEB$color",
	"--color", "BACK$color",
	"-t UPS Data :: $_[0]",
	"DEF:load=$log/$dbf.rrd:load:AVERAGE",
	"DEF:input=$log/$dbf.rrd:input:AVERAGE",
	"DEF:charge=$log/$dbf.rrd:charge:AVERAGE",
	"LINE1:load$color_avg1:Load\\:",
	"GPRINT:load:LAST:currently %.1lf ",
	"LINE1:input$color_avg2:Input\\:",
	"GPRINT:input:LAST:currently %.1lf ",
	"LINE1:charge$color_avg3:Charge\\:",
	"GPRINT:charge:LAST:currently %.1lf\\j");
    $ERROR = RRDs::error;
    print "Error: RRDs::graph failed for '$_[0]' : $ERROR\n" if $ERROR;
}
########################################################################
create "$dbf";
gather;
update "$dbf";
graph 'day',   'Daily';
graph 'week',  'Weekly';
graph 'month', 'Monthly';
graph 'year',  'Yearly';
########################################################################
# vim: sw=4 ts=8
# End
